package scheme.interp;

import scheme.util.*;

public class Store {
    private int location;
    private Env env;
    
    public Store() {
        this(0, new Env());
    }
    
    protected Store(int location, Env env) {
        this.location = location;
        this.env = env;
    }
    
    private void add(SchemeValue v) {
        this.env = this.env.add("" + this.location, v);
    }
    
    private void addAllVoid(List ls) {
        this.env = this.env.addAllVoid(ls);
    }
    
    public int alloc(SchemeValue v) {
        this.location++;
        this.add(v);
        
        return this.location;
    }
    
    public Store allocN(int n) {
        int loop;
        List ls = new EmptyL();
        Store ns = new Store(this.location + n, this.env);
        
        for (loop = 0; loop < n; loop++) {
            ls = ls.snoc("" + this.location + loop);
        }
        
        ns.addAllVoid(ls);
        
        return ns;
    }
    
    public void update(int location, SchemeValue v) throws VisitE {
        this.env.set("" + location, v);
    }
    
    public SchemeValue find(int location) throws VisitE {
        return (SchemeValue)this.env.find("" + location);
    }

// modeled from this Ocaml source by Amr Sabry
// the signature has slightly changed due to the different
// nature of functional vs. imperative programming style
/* 
module type STORE = sig
  type store 
  val empty : store
  val alloc : rtv -> store -> (location * store)
  val allocN : int -> store -> (location list * store)
  val update : location -> rtv -> store -> store
  val find : location -> store -> rtv
end 

module Store : STORE = struct
  type location = int 
  (* the first component is the next free location *)
  (* the second component is a map from locations to values *)
  type store = location * rtv StoreM.t
  
  (* returns a new store *)
  let empty = (0, StoreM.empty)

  (* Allocate and initialize a pointer to v, return the pointer and the new store *)
  let alloc v (fp,sm) = 
    let ns = (fp+1, StoreM.add fp v sm) in
    (fp, ns)

  (* Allocate n pointers initialized to VoidV *)
  (* returns a list of the pointers and the new store *)
  let rec allocN n s = 
    if n = 0 
    then ([],s)
    else let (p1,s1) = alloc VoidV s in
         let (ps,sf) = allocN (n-1) s1 in
         (p1::ps, sf)

  (* Updates an existing location in the store; returns the new store *)
  let update loc v (fp,sm) = (fp, StoreM.add loc v sm)

  (* Finds the value associated with a location *)
  let find loc (_,sm) = StoreM.find loc sm
end 
*/
}
